/*
	webexp.js contains functionality to extract the Prolific PID, create files and store data on a server.
	Author: Daniel Brand (daniel.brand@metech.tu-chemnitz.de)
*/

// Extracts the Prolific ID from the URL.
// Per default, it is called "PROLIFIC_PID"
// If successful, the ID is stored in sessionStorage ("personId") and returned. Otherwise, null is returned
function getProlificPid(paramName='PROLIFIC_PID') {
	let queryString = window.location.search;
	let urlParams = new URLSearchParams(queryString);
	let givenUid = null;
	if(urlParams.has(paramName)) {
		givenUid = urlParams.get(paramName);
		sessionStorage.setItem("personId", givenUid);
	}
	return givenUid;
}

// Saves a given payload and executes a given callback function on completion.
// Alternatively, in case of an error, the errorCallback function is executed.
// If no personId is given (null), then the personId from sessionStorage is used.
// Please check if the php-path is correct relative to the html file
function writeData(payload, callback, errorCallback, 
				personId=null, phpPath='storage/save_data.php') {
	if(personId == null) {
		personId = sessionStorage.getItem("personId");
	}
	if (personId == null) {
		console.error('No person ID found');
		errorCallback();
		return;
	}

	let storagePath = `${phpPath}?id=${personId}`;
	fetch(storagePath, {
	  method: 'POST',
	  headers: {
		'Content-Type': 'application/json',
	  },
	  body: JSON.stringify(payload),
	}).then(response => {
		if (!response.ok) {
		  throw new Error('Could not reach storage server');
		}
		return response.json();
	}).then(data => {
		if(data) {
			callback();
		} else {
			throw new Error('Could not save data');
		}
	}).catch(error => {
		console.error('There has been a problem with your fetch operation:', error);
		errorCallback();
	});
}

// Creates a new file for a person. 
// If given (i.e., the PROLIFIC_PID), the file will be named like that.
// If no personId given, a random identifier will be generated.
// In both cases, the sessionStorage entry for personId will be updated hold the correct identifier.
function createFileForPerson(callback, errorCallback, personId=null, phpPath='storage/create_person.php') {
	if(personId == null) {
		retrieveId(callback, errorCallback, phpPath);
	} else {
		prepareData(personId, callback, errorCallback, phpPath);
	}
}

function retrieveId(callback, errorCallback, phpPath) {
	fetch(phpPath)
	.then(response => {
		if (!response.ok) {
		throw new Error('Could not reach storage server');
		}
		return response.json();
	})
	.then(data => {
		if(data == 0) {
			throw new Error('Server could not create ID');
		}
		
		console.log("Created new file for person without ID: ", data);
		sessionStorage.setItem("personId", data);
		callback();
	})
	.catch(error => {
		console.error('There has been a problem with your fetch operation:', error);
		errorCallback();
	});
}

function prepareData(name, callback, errorCallback, phpPath) {
	let call = `${phpPath}?id=${name}`;
	fetch(call)
	.then(response => {
		if (!response.ok) {
		throw new Error('Could not reach storage server');
		}
		return response.json();
	})
	.then(data => {
		if(data == 0) {
			throw new Error('Server could not create ID');
		}
		
		console.log("Created file for person with ID: ", data);
		sessionStorage.setItem("personId", data);
		callback();
	})
	.catch(error => {
		console.error('There has been a problem with your fetch operation:', error);
		errorCallback();
	});
}